#!/usr/bin/perl
use strict;
use SOAP::Lite;
use Getopt::Long;
use soap_ext;
use vboxService;  

sub print_usage {
   print("Usage:\n$0 [--user=<username>] [--pass=<password>] [--early_unlock] <vm_uuid> <snapshot_name>\n");
   print("\n");
   print("If --early_unlock is specified, the API to unlock the session will be called immediately,\n");
   print("rather than waiting for the IProgress object to indicate completion.\n");
}

sub get_machine {
   our $vbws;
   my $target_uuid = shift;
   my @machineids = vboxService->IVirtualBox_getMachines($vbws);
   foreach my $machine (@machineids) {
      my $machineuuid = vboxService->IMachine_getId($machine);
      if ($machineuuid eq $target_uuid) {
         return $machine;
      }
   }
   return undef;
}
$| = 1; # Auto-flush after every print, regardless of presence of EOL

my $user = "";
my $pass = "";
my $early_unlock = 0;
GetOptions("user=s" => \$user, "pass=s" => \$pass, "early_unlock" => \$early_unlock);
my ($uuid, $snapname) = @ARGV;
if(not defined $uuid) {
   print_usage();
   exit(-1);
}
if(not defined $snapname) {
   print_usage();
   exit(-1);
}
print("Connecting to webservice... ");
#select()->flush();
our $vbws = vboxService->IWebsessionManager_logon($user, $pass);
if(!$vbws) {
   print(" Failed.\n");
   print("Unable to connect.  Maybe the credentials are wrong?\n");
   exit(1);
}
print("  Success.\n");
print("Looking for UUID $uuid... ");
my $machine = get_machine($uuid);
if(not defined $machine) {
   print(" Not found.\n");
   print("Check your UUID?\n");
   exit(-1);
}
print(" Found $machine.\n");
print("Getting session... ");
my $session = vboxService->IWebsessionManager_getSessionObject($machine);
if(not defined $session) {
   print(" Failed.\n");
   print("Maybe the machine is in the wrong state?\n");
   exit(-1);
}
print(" Done.\n");
print("Locking session for shared use... ");
vboxService->IMachine_lockMachine($machine, $session, 'Shared');
print("\nHopefully that worked.  Getting a result code from SOAP is a PITA.\n");
print("Getting a mutable machine reference... ");
my $mutable = vboxService->ISession_getMachine($session);
if(not defined $mutable) {
   print(" Failed.\n");
   print("Not sure what happened in this case!\n");
   vboxService->IWebsessionManager_logoff($vbws);
   exit(-1);
}
print(" Done.\n");
print("Starting snapshot process... ");
my ($snapid, $snapprog) = vboxService->IMachine_takeSnapshot($mutable, $snapname, "", '0');
if(not defined $snapid|| !($snapprog > 0)) {
   print(" Error.\n");
   print("Something went wrong trying to start the snapshot.\n");
   vboxService->ISession_unlockMachine($session);
   vboxService->IWebsessionManager_logoff($vbws);
   exit(-1);
}
print(" Done.\n");
if($early_unlock > 0) {
   print("--early_unlock specified; calling unlock API now!\n");
   vboxService->ISession_unlockMachine($session);
}
print("Checking progress every second...\n");
while(!vboxService->IProgress_getCompleted($snapprog)) {
   printf("Operation: %s\n", vboxService->IProgress_getOperationDescription($snapprog));
   printf("Operation progress: %d %%\n", vboxService->IProgress_getOperationPercent($snapprog));
   printf("Total progress:     %d %%\n", vboxService->IProgress_getPercent($snapprog));
   sleep(3);
}
print("IProgress reports completion.\n");
if($early_unlock == 0) {
   print("Unlocking machine...\n");
   vboxService->ISession_unlockMachine($session);
}
print("Disconnecting...\n");
vboxService->IWebsessionManager_logoff($vbws);

